<?php

namespace Drupal\wp_publish\Commands;

use Consolidation\OutputFormatters\StructuredData\RowsOfFields;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\wp_publish\Batch\WpPublishBatch;
use Drupal\wp_publish\Service\WpPublishService;
use Drush\Commands\DrushCommands;

/**
 * Drush commands for WordPress Publish module.
 */
class WpPublishCommands extends DrushCommands {

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The WordPress publishing service.
   *
   * @var \Drupal\wp_publish\Service\WpPublishService
   */
  protected $publishService;

  /**
   * Constructs a WpPublishCommands object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   * @param \Drupal\wp_publish\Service\WpPublishService $publish_service
   *   The WordPress publishing service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, WpPublishService $publish_service) {
    parent::__construct();
    $this->configFactory = $config_factory;
    $this->publishService = $publish_service;
  }

  /**
   * Publishes nodes to WordPress in batch.
   *
   * @param string $content_type
   *   The content type machine name.
   *
   * @command wp-publish:bulk
   * @aliases wpb
   * @usage wp-publish:bulk article
   */
  public function bulkPublish($content_type) {
    $config = $this->configFactory->get('wp_publish.settings');
    $enabled_types = $config->get('enabled_content_types') ?? [];

    if (!in_array($content_type, $enabled_types)) {
      $this->output()->writeln("Content type $content_type is not enabled for publishing.");
      return;
    }

    $query = \Drupal::entityQuery('node')
      ->condition('type', $content_type)
      ->condition('status', 1)
      ->condition('wp_publish', 1);
    $nids = $query->execute();

    if (empty($nids)) {
      $this->output()->writeln("No nodes found for content type $content_type.");
      return;
    }

    $operations = [];
    foreach ($nids as $nid) {
      $operations[] = [
        [WpPublishBatch::class, 'publishNode'],
        [$nid, $this->publishService],
      ];
    }

    $batch = [
      'title' => t('Publishing nodes to WordPress'),
      'operations' => $operations,
      'finished' => [WpPublishBatch::class, 'finish'],
    ];

    batch_set($batch);
    drush_backend_batch_process();
  }
}
