<?php

namespace Drupal\wp_publish\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configuration form for WordPress Publish settings.
 */
class WpPublishSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'wp_publish_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['wp_publish.settings'];
  }

  /**
   * Builds the form for configuring WordPress connection and field mappings.
   *
   * @param array $form
   *   The form structure.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The form structure.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('wp_publish.settings');

    $form['wordpress_url'] = [
      '#type' => 'url',
      '#title' => $this->t('WordPress Site URL'),
      '#default_value' => $config->get('wordpress_url'),
      '#required' => TRUE,
    ];

    $form['wordpress_username'] = [
      '#type' => 'textfield',
      '#title' => $this->t('WordPress Username'),
      '#default_value' => $config->get('wordpress_username'),
      '#required' => TRUE,
    ];

    $form['wordpress_app_password'] = [
      '#type' => 'password',
      '#title' => $this->t('WordPress Application Password'),
      '#default_value' => $config->get('wordpress_app_password'),
      '#required' => TRUE,
    ];

    $form['enabled_content_types'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Enable for Content Types'),
      '#options' => array_combine(
        array_keys(node_type_get_names()),
        node_type_get_names()
      ),
      '#default_value' => $config->get('enabled_content_types') ?? [],
    ];

    // Field mapping section.
    $form['field_mapping'] = [
      '#type' => 'details',
      '#title' => $this->t('Field Mapping'),
      '#open' => TRUE,
    ];

    $node_fields = \Drupal::service('entity_field.manager')->getFieldDefinitions('node', 'article');
    $field_options = [];
    foreach ($node_fields as $field_name => $field) {
      $field_options[$field_name] = $field->getLabel();
    }

    $form['field_mapping']['title_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Drupal Field for WordPress Title'),
      '#options' => $field_options,
      '#default_value' => $config->get('field_mapping.title_field') ?? 'title',
      '#required' => TRUE,
    ];

    $form['field_mapping']['content_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Drupal Field for WordPress Content'),
      '#options' => $field_options,
      '#default_value' => $config->get('field_mapping.content_field') ?? 'body',
      '#required' => TRUE,
    ];

    $form['field_mapping']['taxonomy_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Drupal Taxonomy Field for WordPress Categories'),
      '#options' => $field_options,
      '#default_value' => $config->get('field_mapping.taxonomy_field') ?? '',
      '#empty_option' => $this->t('- None -'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * Saves the WordPress connection settings and field mappings.
   *
   * @param array $form
   *   The form structure.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('wp_publish.settings')
      ->set('wordpress_url', $form_state->getValue('wordpress_url'))
      ->set('wordpress_username', $form_state->getValue('wordpress_username'))
      ->set('wordpress_app_password', $form_state->getValue('wordpress_app_password'))
      ->set('enabled_content_types', array_filter($form_state->getValue('enabled_content_types')))
      ->set('field_mapping.title_field', $form_state->getValue('title_field'))
      ->set('field_mapping.content_field', $form_state->getValue('content_field'))
      ->set('field_mapping.taxonomy_field', $form_state->getValue('taxonomy_field'))
      ->save();

    parent::submitForm($form, $form_state);
  }
}
