<?php

namespace Drupal\wp_publish\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use GuzzleHttp\ClientInterface;

/**
 * Service for publishing Drupal nodes to WordPress via REST API.
 */
class WpPublishService {

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Constructs a WpPublishService object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger channel factory.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    ClientInterface $http_client,
    LoggerChannelFactoryInterface $logger_factory
  ) {
    $this->configFactory = $config_factory;
    $this->httpClient = $http_client;
    $this->logger = $logger_factory->get('wp_publish');
  }

  /**
   * Publishes a node to WordPress.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The node to publish.
   *
   * @return string|null
   *   The response body or null on failure.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   *   If the API request fails.
   */
  public function publishNode($node) {
    $config = $this->configFactory->get('wp_publish.settings');
    $wordpress_url = $config->get('wordpress_url');
    $username = $config->get('wordpress_username');
    $app_password = $config->get('wordpress_app_password');
    $title_field = $config->get('field_mapping.title_field') ?? 'title';
    $content_field = $config->get('field_mapping.content_field') ?? 'body';
    $taxonomy_field = $config->get('field_mapping.taxonomy_field') ?? '';

    if (!$wordpress_url || !$username || !$app_password) {
      $this->logger->error('WordPress configuration is incomplete.');
      return NULL;
    }

    $post_data = [
      'title' => $title_field === 'title' ? $node->getTitle() : $node->get($title_field)->value,
      'content' => $node->hasField($content_field) ? $node->get($content_field)->value : '',
      'status' => 'publish',
    ];

    // Add taxonomy terms as categories.
    if ($taxonomy_field && $node->hasField($taxonomy_field)) {
      $terms = $node->get($taxonomy_field)->referencedEntities();
      $category_ids = [];
      foreach ($terms as $term) {
        $category_id = $this->getOrCreateCategory($term->getName(), $wordpress_url, $username, $app_password);
        if ($category_id) {
          $category_ids[] = $category_id;
        }
      }
      if (!empty($category_ids)) {
        $post_data['categories'] = $category_ids;
      }
    }

    // Handle media fields.
    $media_fields = ['field_image', 'field_media'];
    foreach ($media_fields as $field_name) {
      if ($node->hasField($field_name)) {
        foreach ($node->get($field_name)->referencedEntities() as $file) {
          if ($file instanceof \Drupal\file\FileInterface) {
            $media_url = $this->uploadMedia($file, $wordpress_url, $username, $app_password);
            if ($media_url) {
              $post_data['content'] .= '<img src="' . $media_url . '" alt="' . $file->getFilename() . '">';
            }
          }
        }
      }
    }

    try {
      $response = $this->httpClient->post($wordpress_url . '/wp-json/wp/v2/posts', [
        'auth' => [$username, $app_password],
        'json' => $post_data,
      ]);

      $this->logger->info('Node @nid published to WordPress.', ['@nid' => $node->id()]);
      return $response->getBody()->getContents();
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to publish node @nid to WordPress: @error', [
        '@nid' => $node->id(),
        '@error' => $e->getMessage(),
      ]);
      return NULL;
    }
  }

  /**
   * Gets or creates a WordPress category by name.
   *
   * @param string $name
   *   The category name.
   * @param string $wordpress_url
   *   The WordPress site URL.
   * @param string $username
   *   The WordPress username.
   * @param string $app_password
   *   The WordPress application password.
   *
   * @return int|null
   *   The category ID or null on failure.
   */
  protected function getOrCreateCategory($name, $wordpress_url, $username, $app_password) {
    try {
      $response = $this->httpClient->get($wordpress_url . '/wp-json/wp/v2/categories', [
        'auth' => [$username, $app_password],
        'query' => ['search' => $name],
      ]);
      $categories = json_decode($response->getBody()->getContents(), TRUE);
      foreach ($categories as $category) {
        if ($category['name'] === $name) {
          return $category['id'];
        }
      }

      $response = $this->httpClient->post($wordpress_url . '/wp-json/wp/v2/categories', [
        'auth' => [$username, $app_password],
        'json' => ['name' => $name],
      ]);
      $category = json_decode($response->getBody()->getContents(), TRUE);
      return $category['id'] ?? NULL;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to get or create category @name: @error', [
        '@name' => $name,
        '@error' => $e->getMessage(),
      ]);
      return NULL;
    }
  }

  /**
   * Uploads a file to the WordPress media library.
   *
   * @param \Drupal\file\FileInterface $file
   *   The file entity.
   * @param string $wordpress_url
   *   The WordPress site URL.
   * @param string $username
   *   The WordPress username.
   * @param string $app_password
   *   The WordPress application password.
   *
   * @return string|null
   *   The media URL or null on failure.
   */
  protected function uploadMedia($file, $wordpress_url, $username, $app_password) {
    try {
      $file_path = \Drupal::service('file_system')->realpath($file->getFileUri());
      $response = $this->httpClient->post($wordpress_url . '/wp-json/wp/v2/media', [
        'auth' => [$username, $app_password],
        'multipart' => [
          [
            'name' => 'file',
            'contents' => fopen($file_path, 'r'),
            'filename' => $file->getFilename(),
          ],
        ],
      ]);
      $media = json_decode($response->getBody()->getContents(), TRUE);
      return $media['source_url'] ?? NULL;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to upload media @fid: @error', [
        '@fid' => $file->id(),
        '@error' => $e->getMessage(),
      ]);
      return NULL;
    }
  }
}
